# Generated by roxygen2: do not edit by hand

S3method("$",chMDB)
S3method("$",fileMDB)
S3method("$",memoMDB)
S3method("$",metaMDB)
S3method("$<-",MDB)
S3method("$<-",TKCat)
S3method("[",TKCat)
S3method("[",chMDB)
S3method("[",fileMDB)
S3method("[",memoMDB)
S3method("[",metaMDB)
S3method("[<-",MDB)
S3method("[<-",TKCat)
S3method("[[",chMDB)
S3method("[[",fileMDB)
S3method("[[",memoMDB)
S3method("[[",metaMDB)
S3method("[[<-",MDB)
S3method("[[<-",TKCat)
S3method("collection_members<-",chMDB)
S3method("collection_members<-",fileMDB)
S3method("collection_members<-",memoMDB)
S3method("db_info<-",chMDB)
S3method("db_info<-",fileMDB)
S3method("db_info<-",memoMDB)
S3method("db_info<-",metaMDB)
S3method("names<-",TKCat)
S3method("names<-",chMDB)
S3method("names<-",fileMDB)
S3method("names<-",memoMDB)
S3method("names<-",metaMDB)
S3method(add_helpers,KMR)
S3method(add_helpers,MDB)
S3method(as.list,MDB)
S3method(as.list,chMDB)
S3method(as.list,fileMDB)
S3method(as.list,metaMDB)
S3method(as_fileMDB,chMDB)
S3method(as_fileMDB,fileMDB)
S3method(as_fileMDB,memoMDB)
S3method(as_fileMDB,metaMDB)
S3method(c,MDB)
S3method(c,TKCat)
S3method(collection_members,TKCat)
S3method(collection_members,chMDB)
S3method(collection_members,chTKCat)
S3method(collection_members,fileMDB)
S3method(collection_members,memoMDB)
S3method(collection_members,metaMDB)
S3method(count_records,MDB)
S3method(data_model,chMDB)
S3method(data_model,fileMDB)
S3method(data_model,memoMDB)
S3method(data_model,metaMDB)
S3method(data_tables,chMDB)
S3method(data_tables,fileMDB)
S3method(data_tables,memoMDB)
S3method(data_tables,metaMDB)
S3method(db_disconnect,DBIConnection)
S3method(db_disconnect,chMDB)
S3method(db_disconnect,chTKCat)
S3method(db_disconnect,metaMDB)
S3method(db_info,chMDB)
S3method(db_info,fileMDB)
S3method(db_info,memoMDB)
S3method(db_info,metaMDB)
S3method(db_reconnect,POK)
S3method(db_reconnect,chMDB)
S3method(db_reconnect,chTKCat)
S3method(db_reconnect,metaMDB)
S3method(dims,chMDB)
S3method(dims,fileMDB)
S3method(dims,memoMDB)
S3method(dims,metaMDB)
S3method(explore_MDBs,TKCat)
S3method(explore_MDBs,chTKCat)
S3method(filter,chMDB)
S3method(filter,fileMDB)
S3method(filter,memoMDB)
S3method(filter,metaMDB)
S3method(filter_mdb_matrix,chMDB)
S3method(filter_mdb_matrix,fileMDB)
S3method(filter_mdb_matrix,memoMDB)
S3method(filter_mdb_matrix,metaMDB)
S3method(filter_with_tables,chMDB)
S3method(filter_with_tables,fileMDB)
S3method(filter_with_tables,memoMDB)
S3method(filter_with_tables,metaMDB)
S3method(format,MDB)
S3method(format,POK)
S3method(format,TKCat)
S3method(format,chTKCat)
S3method(get_MDB,TKCat)
S3method(get_MDB,chTKCat)
S3method(get_R_helpers,KMR)
S3method(get_R_helpers,MDB)
S3method(get_hosts,DBIConnection)
S3method(get_hosts,chMDB)
S3method(get_hosts,chTKCat)
S3method(get_hosts,metaMDB)
S3method(get_query,chMDB)
S3method(get_query,chTKCat)
S3method(heads,chMDB)
S3method(heads,fileMDB)
S3method(heads,memoMDB)
S3method(heads,metaMDB)
S3method(length,MDB)
S3method(lengths,MDB)
S3method(list_MDBs,TKCat)
S3method(list_MDBs,chTKCat)
S3method(list_tables,DBIConnection)
S3method(list_tables,chTKCat)
S3method(merge,MDB)
S3method(names,MDB)
S3method(print,MDB)
S3method(print,POK)
S3method(print,TKCat)
S3method(print,chTKCat)
S3method(pull,MDB)
S3method(rename,TKCat)
S3method(rename,chMDB)
S3method(rename,fileMDB)
S3method(rename,memoMDB)
S3method(rename,metaMDB)
S3method(search_MDB_fields,TKCat)
S3method(search_MDB_fields,chTKCat)
S3method(search_MDB_tables,TKCat)
S3method(search_MDB_tables,chTKCat)
S3method(select,MDB)
S3method(slice,chMDB)
S3method(slice,fileMDB)
S3method(slice,memoMDB)
S3method(slice,metaMDB)
S3method(str,MDB)
export("collection_members<-")
export("db_info<-")
export(MDBs)
export(TKCat)
export(add_chMDB_user)
export(add_chTKCat_collection)
export(add_collection_member)
export(add_feature_def)
export(add_helpers)
export(add_km_feature)
export(add_km_spec)
export(add_km_table)
export(add_property_values)
export(add_table_def)
export(add_table_features)
export(add_unit_def)
export(archive_chMDB)
export(as_KMR)
export(as_chMDB)
export(as_fileMDB)
export(as_memoMDB)
export(chMDB)
export(chTKCat)
export(ch_insert)
export(change_chTKCat_password)
export(check_chTKCat)
export(collection_members)
export(compare_MDB)
export(count_records)
export(create_KMR)
export(create_POK)
export(create_chMDB)
export(create_chTKCat_user)
export(data_file_size)
export(data_files)
export(data_model)
export(data_tables)
export(db_disconnect)
export(db_info)
export(db_reconnect)
export(db_tables)
export(decode_bin)
export(dims)
export(drop_chMDB)
export(drop_chMDB_projections)
export(drop_chTKCat_user)
export(empty_chMDB)
export(encode_bin)
export(explore_MDBs)
export(fileMDB)
export(filter_mdb_matrix)
export(filter_with_tables)
export(get_KMR)
export(get_MDB)
export(get_POK)
export(get_R_helpers)
export(get_chMDB_metadata)
export(get_chMDB_timestamps)
export(get_chTKCat_collection)
export(get_collection_mapper)
export(get_confrontation_report)
export(get_hosts)
export(get_km_spec)
export(get_local_collection)
export(get_query)
export(get_shared_collections)
export(has_km_spec)
export(heads)
export(import_collection_mapper)
export(import_local_collection)
export(init_chTKCat)
export(is.KMR)
export(is.MDB)
export(is.POK)
export(is.TKCat)
export(is.chMDB)
export(is.chTKCat)
export(is.fileMDB)
export(is.memoMDB)
export(is.metaMDB)
export(is_chMDB_public)
export(is_current_chMDB)
export(join_mdb_tables)
export(list_MDBs)
export(list_POKs)
export(list_chMDB_timestamps)
export(list_chMDB_users)
export(list_chTKCat_collections)
export(list_chTKCat_users)
export(list_feature_properties)
export(list_local_collections)
export(list_measurement_units)
export(list_measurements)
export(list_property_values)
export(list_table_features)
export(list_table_types)
export(list_tables)
export(manage_chTKCat_users)
export(map_collection_members)
export(materialize_chMDB_projections)
export(memoMDB)
export(mergeTree_from_RelTableModel)
export(mergeTrees_from_RelDataModel)
export(metaMDB)
export(parse_R_helpers)
export(read_KMR)
export(read_collection_members)
export(read_fileMDB)
export(relational_tables)
export(remove_chMDB_user)
export(remove_chTKCat_collection)
export(rm_km_feature)
export(rm_km_table)
export(scan_fileMDBs)
export(search_MDB_fields)
export(search_MDB_tables)
export(set_chMDB_access)
export(set_chMDB_timestamp)
export(show_collection_def)
export(unarchive_chMDB)
export(update_chMDB_grants)
export(update_chTKCat_user)
export(write_MergeTree)
export(write_collection_members)
import(DBI)
import(ReDaMoR)
import(crayon)
import(dplyr)
import(promises)
import(shiny)
import(shinydashboard)
import(visNetwork)
importFrom(ClickHouseHTTP,ClickHouseHTTP)
importFrom(Matrix,drop0)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,summary)
importFrom(askpass,askpass)
importFrom(future,future)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(graphics,plot)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,read_json)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,unbox)
importFrom(jsonvalidate,json_validate)
importFrom(markdown,renderMarkdown)
importFrom(readr,DataFrameCallback)
importFrom(readr,read_delim)
importFrom(readr,read_delim_chunked)
importFrom(readr,write_tsv)
importFrom(rlang,.data)
importFrom(rlang,enquo)
importFrom(rlang,set_names)
importFrom(roxygen2,parse_text)
importFrom(stats,setNames)
importFrom(tidyselect,eval_rename)
importFrom(tidyselect,eval_select)
importFrom(tidyselect,vars_pull)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,packageName)
importFrom(utils,read.delim)
importFrom(utils,str)
importFrom(utils,write.table)
importFrom(utils,zip)
importFrom(uuid,UUIDgenerate)
importFrom(xml2,read_xml)
importFrom(xml2,write_xml)
importFrom(xml2,xml_children)
importFrom(xml2,xml_name)
importFrom(xml2,xml_replace)
